@@ -1,9 +1,14 @@
 # The R Commander and command logger
 
-# last modified 1 December 06 by J. Fox
-#   slight changes 12 Aug 04 by Ph. Grosjean 
+# last modified 1 June 2008 by J. Fox
+#   slight changes 12 Aug 04 by Ph. Grosjean
+#   changes 21 June 2007 by Erich Neuwirth for Excel support (marked EN)
 
 Commander <- function(){
+    RcmdrVersion <- "1.3-15"
+##    DESCRIPTION <- readLines(file.path(.find.package("Rcmdr"), "DESCRIPTION")[1])
+##    RcmdrVersion <- trim.blanks(sub("^Version:", "",
+##        grep("^Version:", D, value=TRUE)))
     # the following test suggested by Richard Heiberger
     if ("RcmdrEnv" %in% search() &&
         exists("commanderWindow", "RcmdrEnv") &&
@@ -11,16 +16,16 @@ Commander <- function(){
       warning("The R Commander is already open.")
       return(invisible(NULL))
     }
-    RcmdrVersion <- "1.2-5"
     if (is.SciViews()) return(invisible(svCommander(Version=RcmdrVersion))) # +PhG
     setOption <- function(option, default, global=TRUE) {
-        opt <- if (is.null(current[[option]])) default else current[[option]]
+        opt <- if (is.null(current[option][[1]])) default else current[option][[1]]
         if (global) putRcmdr(option, opt)
         else opt
         }
     current <- options("Rcmdr")[[1]]
-    etc <- setOption("etc", file.path(.path.package(package="Rcmdr")[1], "etc"), global=FALSE)
-    etcMenus <- setOption("etcMenus", etc, global=FALSE)
+    etc <- setOption("etc", file.path(.path.package(package="Rcmdr")[1], "etc"))
+    etcMenus <- setOption("etcMenus", etc)
+    putRcmdr("etcMenus", etcMenus)
     onCopy <- function(){
         focused <- tkfocus()
         if ((tclvalue(focused) != LogWindow()$ID) && (tclvalue(focused) != OutputWindow()$ID))
@@ -58,9 +63,9 @@ Commander <- function(){
         initializeDialog(title=gettextRcmdr("Find"))
         textFrame <- tkframe(top)
         textVar <- tclVar("")
-        textEntry <- tkentry(textFrame, width="20", textvariable=textVar)
+        textEntry <- ttkentry(textFrame, width="20", textvariable=textVar)
         checkBoxes(frame="optionsFrame", boxes=c("regexpr", "case"), initialValues=c("0", "1"),
-            labels=gettextRcmdr(c("Regular-expression search", "Case sensitive"))) 
+            labels=gettextRcmdr(c("Regular-expression search", "Case sensitive")))
         radioButtons(name="direction", buttons=c("foward", "backward"), labels=gettextRcmdr(c("Forward", "Backward")),
             values=c("-forward", "-backward"), title=gettextRcmdr("Search Direction"))
         onOK <- function(){
@@ -91,7 +96,7 @@ Commander <- function(){
             tkdestroy(top)
             }
         OKCancelHelp()
-        tkgrid(tklabel(textFrame, text=gettextRcmdr("Search for:")), textEntry, sticky="w")
+        tkgrid(labelRcmdr(textFrame, text=gettextRcmdr("Search for:")), textEntry, sticky="w")
         tkgrid(textFrame, sticky="w")
         tkgrid(optionsFrame, sticky="w")
         tkgrid(directionFrame, sticky="w")
@@ -121,11 +126,11 @@ Commander <- function(){
     putRcmdr("Identify3d", NULL)
     setOption("log.font.size", if (.Platform$OS.type == "windows") 10 else 12)
     putRcmdr("logFont", tkfont.create(family="courier", size=getRcmdr("log.font.size")))
-    putRcmdr("operatorFont", tkfont.create(family="courier", size=getRcmdr("log.font.size")))
+#    putRcmdr("operatorFont", tkfont.create(family="courier", size=getRcmdr("log.font.size")))
     scale.factor <- current$scale.factor
     if (!is.null(scale.factor)) .Tcl(paste("tk scaling ", scale.factor, sep=""))
     if (packageAvailable("car")){
-        require("car") 
+        require("car")
         setOption("contrasts", c("contr.Treatment", "contr.poly"))
         }
     else setOption("contrasts", c("contr.treatment", "contr.poly"))
@@ -139,6 +144,7 @@ Commander <- function(){
         else 20, global=FALSE))
     putRcmdr("saveOptions", options(warn=1, contrasts=getRcmdr("contrasts"), width=as.numeric(log.width),
         na.action="na.exclude", graphics.record=TRUE))
+    setOption("ask.on.exit", TRUE)
     setOption("double.click", FALSE)
     setOption("sort.names", TRUE)
     setOption("grab.focus", TRUE)
@@ -149,11 +155,14 @@ Commander <- function(){
     setOption("error.text.color", "red")
     setOption("warning.text.color", "darkgreen")
     setOption("multiple.select.mode", "extended")
-    setOption("suppress.X11.warnings", 
+    setOption("suppress.X11.warnings",
         interactive() && .Platform$GUI == "X11" && getRversion() < "2.4.0") # to address problem in Linux
     setOption("showData.threshold", 100)
     setOption("retain.messages", TRUE)
-    setOption("crisp.dialogs",  (.Platform$OS.type == "windows") && (getRversion() >= "2.1.1"))
+    setOption("crisp.dialogs",  TRUE)
+    setOption("length.output.stack", 10)
+    putRcmdr("outputStack", as.list(rep(NA, getRcmdr("length.output.stack"))))
+    setOption("variable.list.height", 4)
     if (getRcmdr("suppress.X11.warnings")) {
         putRcmdr("messages.connection", file(open = "w+"))
         sink(getRcmdr("messages.connection"), type="message")
@@ -166,59 +175,96 @@ Commander <- function(){
             paste("*helvetica-medium-r-normal-*-", default.font.size, "*", sep=""), global=FALSE)
         .Tcl(paste("option add *font ", default.font, sep=""))
         }
-    if (getRcmdr("crisp.dialogs")) tclServiceMode(on=FALSE)
-    putRcmdr("commanderWindow", tktoplevel())
-    .commander <- CommanderWindow()
     placement <- setOption("placement", "-40+20", global=FALSE)
-    tkwm.geometry(.commander, placement)
-    tkwm.title(.commander, gettextRcmdr("R Commander"))
-    tkwm.protocol(.commander, "WM_DELETE_WINDOW", closeCommander)
-    topMenu <- tkmenu(.commander)
-    tkconfigure(.commander, menu=topMenu)
     source.files <- list.files(etc, pattern="\\.[Rr]$")
     for (file in source.files) {
         source(file.path(etc, file))
         cat(paste(gettextRcmdr("Sourced:"), file, "\n"))
         }
-    Menus <- read.table(file.path(etcMenus, "Rcmdr-menus.txt"), as.is=TRUE)
+    Plugins <- options()$Rcmdr$plugins
+    allPlugins <- listPlugins(loaded=TRUE)
+    for (plugin in Plugins){
+        if (!require(plugin, character.only=TRUE)){
+            putRcmdr("commanderWindow", NULL)
+            stop(sprintf(gettextRcmdr("the plug-in package %s is missing"), plugin))
+            }
+        if (!is.element(plugin, allPlugins)){
+            putRcmdr("commanderWindow", NULL)
+            stop(sprintf(gettextRcmdr("the package %s is not an Rcmdr plug-in"), plugin))
+            }
+        }
+    Menus <- read.table(file.path(etcMenus, "Rcmdr-menus.txt"), colClasses = "character")
+    addMenus <- function(Menus){
+        nms <- c("type", "menuOrItem", "operationOrParent", "label",
+            "commandOrMenu", "activation", "install")
+        names(Menus) <- nms
+        for (plugin in Plugins) {
+            MenusToAdd <- read.table(file.path(.path.package(package=plugin)[1], "etc/menus.txt"),
+                colClasses = "character")
+            names(MenusToAdd) <- nms
+            for (i in 1:nrow(MenusToAdd)){
+                line <- MenusToAdd[i,]
+                if (line[1, "type"] == "menu"){
+                    where <- if (line[1, "operationOrParent"] == "topMenu") 0
+                        else which((Menus[, "type"] == "menu") &
+                            (Menus[, "menuOrItem"] == line[1, "operationOrParent"]))
+                    }
+                else if (line[1, "type"] == "item"){
+                    if (line[1, "operationOrParent"] == "command"){
+                        which <- which((Menus[, "operationOrParent"] == "command") &
+                            (Menus[, "menuOrItem"] == line[1, "menuOrItem"]))
+                        where <- if (length(which) == 0)
+                            which((Menus[, "type"] == "menu")
+                                & (Menus[, "menuOrItem"] == line[1, "menuOrItem"]))
+                            else max(which)
+                        }
+                    else if (line[1, "operationOrParent"] == "cascade"){
+                        where <- if (line[1, "menuOrItem"] != "topMenu")
+                            max(which((Menus[, "operationOrParent"] == "cascade") &
+                                (Menus[, "commandOrMenu"] == line[1, "menuOrItem"])))
+                        else {
+                            max(which((Menus[, "operationOrParent"] == "cascade") &
+                                (Menus[, "menuOrItem"] == "topMenu") &
+                                (Menus[, "commandOrMenu"] != "toolsMenu") &
+                                (Menus[, "commandOrMenu"] != "helpMenu")))
+                            }
+                        }
+                    else stop(sprintf(gettextRcmdr('unrecognized operation, "%s", in plugin menu line %i'),
+                        line[1, "operation"], i))
+                    }
+                else stop(sprintf(gettextRcmdr('unrecognized type, "%s", in plugin menu line %i'),
+                   line[1, "type"], i))
+                Menus <- insertRows(Menus, line, where)
+                }
+            }
+        Menus
+        }
+    Menus <- addMenus(Menus)
     .Menus <- menus <- list()
     menuItems <- 0
     oldMenu <- ncol(Menus) == 6
-    for (m in 1:nrow(Menus)){
-        install <- if (oldMenu) "" else Menus[m, 7]
-        if ((install != "") && (!eval(parse(text=install)))) next  
-        if (Menus[m, 1] == "menu") {
-            position <- 0
-            assign(Menus[m, 2], tkmenu(eval(parse(text=Menus[m, 3])), tearoff=FALSE))
-            menus[[Menus[m, 2]]] <- list(ID=get(Menus[m, 2])$ID, position=0)
-            }
-        else if (Menus[m, 1] == "item") {
-            if (Menus[m, 3] == "command"){
-                position <- position + 1
-                if (Menus[m, 6] == "")
-                    tkadd(eval(parse(text=Menus[m, 2])),"command", label=gettextRcmdr(Menus[m, 4]),
-                        command=eval(parse(text=Menus[m, 5])))
-                else {
-                    tkadd(eval(parse(text=Menus[m, 2])),"command", label=gettextRcmdr(Menus[m, 4]),
-                        command=eval(parse(text=Menus[m, 5])),  state="disabled")
-                    menuItems <- menuItems + 1
-                    menus[[Menus[m, 2]]]$position <- position
-                    .Menus[[menuItems]] <- list(ID=menus[[Menus[m, 2]]]$ID, position=position,
-                        activation=eval(parse(text=paste("function()", Menus[m, 6]))))
-                    }
-                }
-            else if (Menus[m, 3] == "cascade")
-                tkadd(eval(parse(text=Menus[m, 2])),"cascade", label=gettextRcmdr(Menus[m, 4]), menu=eval(parse(text=Menus[m, 5])))
-            else stop(paste(gettextRcmdr("menu definition error:"), Menus[m, ], collapse=" "),
-                domain=NA)
-            }
-        else stop(paste(gettextRcmdr("menu definition error:"), Menus[m, ], collapse=" "),
-            domain=NA)
+    setOption("suppress.menus", FALSE)
+    ## added by EN ###############################
+	if (RExcelSupported())
+    	putRExcel(".rexcel.menu.dataframe", Menus)
+    ## end of change ###############################
+##    exceptions <- scan(file.path(etc, "log-exceptions.txt"), what="", quiet=TRUE, comment.char="#")
+    ## added by EN ###############################
+##   	putRcmdr("Commander.Input.exceptions", exceptions)
+    ## end of change ###############################
+    modelClasses <- scan(file.path(etc, "model-classes.txt"), what="", quiet=TRUE, comment.char="#")
+    for (plugin in Plugins){
+        description <- readLines(file.path(.path.package(package=plugin)[1], "DESCRIPTION"))
+##        addExceptions <- description[grep("Log-Exceptions:", description)]
+##        addExceptions <- gsub(" ", "", sub("^Log-Exceptions:", "", addExceptions))
+##        addExceptions <- unlist(strsplit(addExceptions, ","))
+        addModels <- description[grep("Models:", description)]
+        addModels <- gsub(" ", "", sub("^Models:", "", addModels))
+        addModels <- unlist(strsplit(addModels, ","))
+##        if (length(addExceptions) > 0) exceptions <- c(exceptions, addExceptions)
+        if (length(addModels) > 0) modelClasses <- c(modelClasses, addModels)
         }
-    putRcmdr("Menus", .Menus)
-    activateMenus()
-    exceptions <- scan(file.path(etc, "log-exceptions.txt"), what="", quiet=TRUE, comment.char="#")
-    putRcmdr("modelClasses", scan(file.path(etc, "model-classes.txt"), what="", quiet=TRUE, comment.char="#"))
+    putRcmdr("modelClasses", modelClasses)
     onEdit <- function(){
         if (activeDataSet() == FALSE) {
             tkfocus(CommanderWindow())
@@ -237,24 +283,26 @@ Commander <- function(){
             return()
             }
         view.height <- max(as.numeric(output.height) + as.numeric(log.height), 10)
-        ncols <- eval(parse(text=paste("ncol(", ActiveDataSet(), ")")))
+        ncols <- ncol(get(ActiveDataSet()))
+#        ncols <- eval(parse(text=paste("ncol(", ActiveDataSet(), ")")))
         command <- if (packageAvailable("relimp") && ncols <= getRcmdr("showData.threshold")){
             require("relimp")
             paste("showData(", ActiveDataSet(), ", placement='-20+200', font=getRcmdr('logFont'), maxwidth=",
                 log.width, ", maxheight=", view.height, ")", sep="")
             }
-        else paste("invisible(edit(", ActiveDataSet(), "))", sep="")
+        else paste("View(", ActiveDataSet(), ")", sep="")
         logger(command)
         justDoIt(command)
         tkwm.deiconify(CommanderWindow())
         tkfocus(CommanderWindow())
         }
+    # the following function modified 14 July 07 by Erich Neuwirth
     onSubmit <- function(){
         .log <- LogWindow()
         selection <- strsplit(tclvalue(tktag.ranges(.log, "sel")), " ")[[1]]
         if (is.na(selection[1])) {
             tktag.add(.log, "currentLine", "insert linestart", "insert lineend")
-            selection <- strsplit(tclvalue(tktag.ranges(.log, "currentLine")), " ")[[1]]
+            selection <- strsplit(tclvalue(tktag.ranges(.log,"currentLine")), " ")[[1]]
             tktag.delete(.log, "currentLine")
             if (is.na(selection[1])) {
                 Message(message=gettextRcmdr("Nothing is selected."),
@@ -267,46 +315,36 @@ Commander <- function(){
         lines <- strsplit(lines, "\n")[[1]]
         .console.output <- getRcmdr("console.output")
         .output <- OutputWindow()
-        if (!.console.output) tkinsert(.output, "end", "\n")
         iline <- 1
         nlines <- length(lines)
         while (iline <= nlines){
+            while (nchar(lines[iline])==0) iline <- iline + 1
+            if (iline > nlines) break
             current.line <- lines[iline]
             if (.console.output) cat(paste("\nRcmdr> ", current.line,"\n", sep=""))
             else{
-                tkinsert(.output, "end", paste("> ", current.line,"\n", sep=""))
+                tkinsert(.output, "end", paste("\n> ", current.line,"\n", sep="")) ### end of changed
                 tktag.add(.output, "currentLine", "end - 2 lines linestart", "end - 2 lines lineend")
                 tktag.configure(.output, "currentLine", foreground=getRcmdr("command.text.color"))
                 }
             jline <- iline + 1
             while (jline <= nlines){
-                if (length(grep("^[\\ \t]", lines[jline])) == 0) break
+                   if (class(try(parse(text=current.line),silent=TRUE))!="try-error") break
                 if (.console.output)cat(paste("Rcmdr+ ", lines[jline],"\n", sep=""))
                 else{
                     tkinsert(.output, "end", paste("+ ", lines[jline],"\n", sep=""))
                     tktag.add(.output, "currentLine", "end - 2 lines linestart", "end - 2 lines lineend")
                     tktag.configure(.output, "currentLine", foreground=getRcmdr("command.text.color"))
                     }
-                current.line <- paste(current.line, lines[jline])
+                current.line <- paste(current.line, lines[jline],sep="\n")
                 jline <- jline + 1
                 iline <- iline + 1
                 }
-            if (length(grep("<-", current.line)) > 0){
-                justDoIt(current.line)
-                }
-            else if (length(grep("^remove\\(", current.line)) > 0){
-                current.line <- sub(")", ", envir=.GlobalEnv)", current.line)
-                justDoIt(current.line)
-                }
-            else if (any(sapply(exceptions,
-                    function(.x) length(grep(paste("^", .x, "\\(", sep=""), current.line)) > 0))){
-                justDoIt(current.line)
-                }
-            else doItAndPrint(current.line, log=FALSE)
+            if (!(is.null(current.line) || is.na(current.line))) doItAndPrint(current.line, log=FALSE)
             iline <- iline + 1
-            }
-        tkyview.moveto(.output, 1)
+            tkyview.moveto(.output, 1)
         }
+    }
     contextMenuLog <- function(){
         .log <- LogWindow()
         contextMenu <- tkmenu(tkmenu(.log), tearoff=FALSE)
@@ -332,36 +370,90 @@ Commander <- function(){
         tkadd(contextMenu, "command", label=gettextRcmdr("Select all"), command=onSelectAll)
         tkpopup(contextMenu, tkwinfo("pointerx", .output), tkwinfo("pointery", .output))
         }
+    if (getRcmdr("crisp.dialogs")) tclServiceMode(on=FALSE)
+    putRcmdr("commanderWindow", tktoplevel())
+    .commander <- CommanderWindow()
+#    tkwm.withdraw(.commander)
+    tkwm.geometry(.commander, placement)
+    tkwm.title(.commander, gettextRcmdr("R Commander"))
+    tkwm.protocol(.commander, "WM_DELETE_WINDOW", CloseCommander)
+    topMenu <- tkmenu(.commander)
+    tkconfigure(.commander, menu=topMenu)
+    if (!getRcmdr("suppress.menus")){
+        for (m in 1:nrow(Menus)){
+            install <- if (oldMenu) "" else Menus[m, 7]
+            if ((install != "") && (!eval(parse(text=install)))) next
+            if (Menus[m, 1] == "menu") {
+                position <- 0
+                assign(Menus[m, 2], tkmenu(get(Menus[m, 3]), tearoff=FALSE))
+#                assign(Menus[m, 2], tkmenu(eval(parse(text=Menus[m, 3])), tearoff=FALSE))
+                menus[[Menus[m, 2]]] <- list(ID=get(Menus[m, 2])$ID, position=0)
+                }
+            else if (Menus[m, 1] == "item") {
+                if (Menus[m, 3] == "command"){
+                    position <- position + 1
+                    if (Menus[m, 6] == "")
+                    tkadd(get(Menus[m, 2]), "command", label=gettextRcmdr(Menus[m, 4]),
+                        command=get(Menus[m, 5]))
+#                        tkadd(eval(parse(text=Menus[m, 2])),"command", label=gettextRcmdr(Menus[m, 4]),
+#                            command=eval(parse(text=Menus[m, 5])))
+                    else {
+                        tkadd(get(Menus[m, 2]), "command", label=gettextRcmdr(Menus[m, 4]),
+                            command=get(Menus[m, 5]), state="disabled")
+#                        tkadd(eval(parse(text=Menus[m, 2])),"command", label=gettextRcmdr(Menus[m, 4]),
+#                            command=eval(parse(text=Menus[m, 5])),  state="disabled")
+                        menuItems <- menuItems + 1
+                        menus[[Menus[m, 2]]]$position <- position
+                        .Menus[[menuItems]] <- list(ID=menus[[Menus[m, 2]]]$ID, position=position,
+                            activation=eval(parse(text=paste("function()", Menus[m, 6]))))
+                        }
+                    }
+                else if (Menus[m, 3] == "cascade")
+                    tkadd(get(Menus[m, 2]), "cascade", label=gettextRcmdr(Menus[m, 4]),
+                        menu=get(Menus[m, 5]))
+#                    tkadd(eval(parse(text=Menus[m, 2])),"cascade", label=gettextRcmdr(Menus[m, 4]),
+#                        menu=eval(parse(text=Menus[m, 5])))
+                else stop(paste(gettextRcmdr("menu definition error:"), Menus[m, ], collapse=" "),
+                    domain=NA)
+                }
+            else stop(paste(gettextRcmdr("menu definition error:"), Menus[m, ], collapse=" "),
+                domain=NA)
+            }
+        }
+    putRcmdr("Menus", .Menus)
+    putRcmdr("autoRestart", FALSE)
+    activateMenus()
     controlsFrame <- tkframe(CommanderWindow())
-    editButton <- tkbutton(controlsFrame, text=gettextRcmdr("Edit data set"), command=onEdit)
-    viewButton <- tkbutton(controlsFrame, text=gettextRcmdr("View data set"), command=onView)
+    editButton <- buttonRcmdr(controlsFrame, text=gettextRcmdr("Edit data set"), command=onEdit)
+    viewButton <- buttonRcmdr(controlsFrame, text=gettextRcmdr("View data set"), command=onView)
     putRcmdr("dataSetName", tclVar(gettextRcmdr("<No active dataset>")))
-    putRcmdr("dataSetLabel", tkbutton(controlsFrame, textvariable=getRcmdr("dataSetName"), fg="red",
+    putRcmdr("dataSetLabel", tkbutton(controlsFrame, textvariable=getRcmdr("dataSetName"), foreground="red",
         relief="groove", command=selectActiveDataSet))
     logFrame <- tkframe(CommanderWindow())
-    putRcmdr("logWindow", tktext(logFrame, bg="white", fg=getRcmdr("log.text.color"),
+    putRcmdr("logWindow", tktext(logFrame, bg="white", foreground=getRcmdr("log.text.color"),
         font=getRcmdr("logFont"), height=log.height, width=log.width, wrap="none"))
     .log <- LogWindow()
-    logXscroll <- tkscrollbar(logFrame, repeatinterval=5, orient="horizontal",
+    logXscroll <- ttkscrollbar(logFrame, orient="horizontal",
         command=function(...) tkxview(.log, ...))
-    logYscroll <- tkscrollbar(logFrame, repeatinterval=5,
+    logYscroll <- ttkscrollbar(logFrame,
         command=function(...) tkyview(.log, ...))
     tkconfigure(.log, xscrollcommand=function(...) tkset(logXscroll, ...))
     tkconfigure(.log, yscrollcommand=function(...) tkset(logYscroll, ...))
     outputFrame <- tkframe(.commander)
+    submitIm <- tcl("image", "create", "bitmap", file=file.path(etc, "submit.xbm"))
     if (getRcmdr("console.output"))
-        submitButton <- if (English()) tkbutton(logFrame, bitmap=paste("@", file.path(etc, "submit.xbm"), sep=""),
+        submitButton <- if (English()) buttonRcmdr(logFrame, image=submitIm,
             borderwidth="2", command=onSubmit)
-        else tkbutton(logFrame, text=gettextRcmdr("Submit"), borderwidth="2", command=onSubmit)
-    else submitButton <- if (English())tkbutton(outputFrame, bitmap=paste("@", file.path(etc, "submit.xbm"), sep=""),
+        else buttonRcmdr(logFrame, text=gettextRcmdr("Submit"), borderwidth="2", command=onSubmit)
+    else submitButton <- if (English()) buttonRcmdr(outputFrame, image=submitIm,
             borderwidth="2", command=onSubmit)
-        else tkbutton(outputFrame, text=gettextRcmdr("Submit"), borderwidth="2", command=onSubmit)
-    putRcmdr("outputWindow", tktext(outputFrame, bg="white", fg=getRcmdr("output.text.color"),
+        else buttonRcmdr(outputFrame, text=gettextRcmdr("Submit"), borderwidth="2", command=onSubmit)
+    putRcmdr("outputWindow", tktext(outputFrame, bg="white", foreground=getRcmdr("output.text.color"),
         font=getRcmdr("logFont"), height=output.height, width=log.width, wrap="none"))
     .output <- OutputWindow()
-    outputXscroll <- tkscrollbar(outputFrame, repeatinterval=5, orient="horizontal",
+    outputXscroll <- ttkscrollbar(outputFrame, orient="horizontal",
         command=function(...) tkxview(.output, ...))
-    outputYscroll <- tkscrollbar(outputFrame, repeatinterval=5,
+    outputYscroll <- ttkscrollbar(outputFrame,
         command=function(...) tkyview(.output, ...))
     tkconfigure(.output, xscrollcommand=function(...) tkset(outputXscroll, ...))
     tkconfigure(.output, yscrollcommand=function(...) tkset(outputYscroll, ...))
@@ -369,35 +461,38 @@ Commander <- function(){
     putRcmdr("messagesWindow", tktext(messagesFrame, bg="lightgray",
         font=getRcmdr("logFont"), height=3, width=log.width, wrap="none"))
     .messages <- MessagesWindow()
-    messagesXscroll <- tkscrollbar(messagesFrame, repeatinterval=5, orient="horizontal",
+    messagesXscroll <- ttkscrollbar(messagesFrame, orient="horizontal",
         command=function(...) tkxview(.messages, ...))
-    messagesYscroll <- tkscrollbar(messagesFrame, repeatinterval=5,
+    messagesYscroll <- ttkscrollbar(messagesFrame,
         command=function(...) tkyview(.messages, ...))
     tkconfigure(.messages, xscrollcommand=function(...) tkset(messagesXscroll, ...))
     tkconfigure(.messages, yscrollcommand=function(...) tkset(messagesYscroll, ...))
     putRcmdr("modelName", tclVar(gettextRcmdr("<No active model>")))
-    putRcmdr("modelLabel", tkbutton(controlsFrame, textvariable=getRcmdr("modelName"), fg="red",
+    putRcmdr("modelLabel", tkbutton(controlsFrame, textvariable=getRcmdr("modelName"), foreground="red",
         relief="groove", command=selectActiveModel))
     show.edit.button <- options("Rcmdr")[[1]]$show.edit.button
     show.edit.button <- if (is.null(show.edit.button)) TRUE else show.edit.button
-    tkgrid(tklabel(controlsFrame, bitmap=paste("@", file.path(etc, "Rcmdr.xbm"), sep=""), fg="red"),
-        tklabel(controlsFrame, text=gettextRcmdr("Data set:")), getRcmdr("dataSetLabel"),
-        tklabel(controlsFrame, text="  "), if(show.edit.button) editButton, viewButton,
-        tklabel(controlsFrame, text=gettextRcmdr("    Model: ")), getRcmdr("modelLabel"), sticky="w")
-    tkgrid(controlsFrame, sticky="w")
+    if (!getRcmdr("suppress.menus")){
+        RcmdrIm <- tcl("image", "create", "bitmap", file=file.path(etc, "Rcmdr.xbm"), foreground="red")
+       tkgrid(labelRcmdr(controlsFrame, image=RcmdrIm),
+            labelRcmdr(controlsFrame, text=gettextRcmdr("Data set:")), getRcmdr("dataSetLabel"),
+            labelRcmdr(controlsFrame, text="  "), if(show.edit.button) editButton, viewButton,
+            labelRcmdr(controlsFrame, text=gettextRcmdr("    Model: ")), getRcmdr("modelLabel"), sticky="w")
+        tkgrid(controlsFrame, sticky="w")
+        }
     .log.commands <-  getRcmdr("log.commands")
     .console.output <- getRcmdr("console.output")
-    if (.log.commands) tkgrid(tklabel(logFrame, text=gettextRcmdr("Script Window"), fg="blue"),
+    if (.log.commands) tkgrid(labelRcmdr(logFrame, text=gettextRcmdr("Script Window"), foreground="blue"),
         if (.log.commands && .console.output) submitButton, sticky="w")
     tkgrid(.log, logYscroll, sticky="news", columnspan=2)
     tkgrid(logXscroll)
     if (.log.commands) tkgrid(logFrame, sticky="news", padx=10, pady=0, columnspan=2)
-    tkgrid(tklabel(outputFrame, text=gettextRcmdr("Output Window"), fg="blue"),
+    tkgrid(labelRcmdr(outputFrame, text=gettextRcmdr("Output Window"), foreground="blue"),
         if (.log.commands && !.console.output) submitButton, sticky="w")
     tkgrid(.output, outputYscroll, sticky="news", columnspan=2)
     tkgrid(outputXscroll, columnspan=1 + (.log.commands && !.console.output))
     if (!.console.output) tkgrid(outputFrame, sticky="news", padx=10, pady=0, columnspan=2)
-    tkgrid(tklabel(messagesFrame, text=gettextRcmdr("Messages"), fg=getRcmdr("error.text.color")), sticky="w")
+    tkgrid(labelRcmdr(messagesFrame, text=gettextRcmdr("Messages"), foreground=getRcmdr("error.text.color")), sticky="w")
     tkgrid(.messages, messagesYscroll, sticky="news", columnspan=2)
     tkgrid(messagesXscroll)
     tkgrid(messagesFrame, sticky="news", padx=10, pady=0, columnspan=2)
@@ -435,6 +530,7 @@ Commander <- function(){
     .Tcl("update idletasks")
     tkbind(.commander, "<Control-r>", onSubmit)
     tkbind(.commander, "<Control-R>", onSubmit)
+    tkbind(.commander, "<Control-Tab>", onSubmit)
     tkbind(.commander, "<Control-f>", onFind)
     tkbind(.commander, "<Control-F>", onFind)
     tkbind(.commander, "<Control-s>", saveLog)
@@ -448,32 +544,43 @@ Commander <- function(){
     if (getRcmdr("crisp.dialogs")) tclServiceMode(on=TRUE)
     tkwait <- options("Rcmdr")[[1]]$tkwait  # to address problem in Debian Linux
     if ((!is.null(tkwait)) && tkwait) {
-        .commander.done <<- tclVar("0") 
+        .commander.done <<- tclVar("0")
         tkwait.variable(.commander.done)
         }
 ##    if (!packageAvailable("rgl")) Message(gettextRcmdr("The rgl package is absent; 3D plots are unavailable."), type="warning")
     Message(paste(gettextRcmdr("R Commander Version "), getRcmdr("RcmdrVersion"), ": ", date(), sep=""))
     }
 
+# the following function modified 24 July 07 by Richard Heiberger
+#  and subsequently by J. Fox 26 July 07
+
 logger <- function(command){
-    if (is.SciViews()) return(svlogger(command))    # +PhG
-    .log <- LogWindow()
-    .output <- OutputWindow()
-    if (getRcmdr("log.commands")) {
-        tkinsert(.log, "end", paste(command,"\n", sep=""))
-        tkyview.moveto(.log, 1)
+   if (is.SciViews()) return(svlogger(command))    # +PhG
+   .log <- LogWindow()
+   .output <- OutputWindow()
+   if (getRcmdr("log.commands")) {
+       tkinsert(.log, "end", paste(command,"\n", sep=""))
+       tkyview.moveto(.log, 1)
+       }
+   lines <- strsplit(command, "\n")[[1]]
+   tkinsert(.output, "end", "\n")
+   if (getRcmdr("console.output")) {
+     for (line in seq(along=lines)) {
+       prompt <- ifelse (line==1, "\nRcmdr>", "\nRcmdr+")
+       cat(paste(prompt, lines[line], "\n"))
         }
-    lines <- strsplit(command, "\n")[[1]]
-    tkinsert(.output, "end", "\n")
-    if (getRcmdr("console.output")) for (line in lines) cat(paste("\nRcmdr>", line, "\n"))
-    else {
-        for (line in lines) tkinsert(.output, "end", paste("> ", command,"\n", sep=""))
-        tktag.add(.output, "currentLine", "end - 2 lines linestart", "end - 2 lines lineend")
-        tktag.configure(.output, "currentLine", foreground=getRcmdr("command.text.color"))
-        tkyview.moveto(.output, 1)
-        }
-    command
-    }
+     }
+   else {
+     for (line in  seq(along=lines)) {
+       prompt <- ifelse(line==1, "> ", "+ ")
+       tkinsert(.output, "end", paste(prompt, lines[line], "\n", sep=""))
+       tktag.add(.output, "currentLine", "end - 2 lines linestart", "end - 2 lines lineend")
+       tktag.configure(.output, "currentLine", foreground=getRcmdr("command.text.color"))
+       tkyview.moveto(.output, 1)
+       }
+     }
+   command
+   }
 
 justDoIt <- function(command) {
     Message()
@@ -497,12 +604,13 @@ justDoIt <- function(command) {
     }
 
 doItAndPrint <- function(command, log=TRUE) {
+    # with modifications from Duncan Murdoch 4 Jan 08
     Message()
     .console.output <- getRcmdr("console.output")
     .output <- OutputWindow()
     if (!.console.output) {
         width <- (as.numeric(tkwinfo("width", .output)) - 2*as.numeric(tkcget(.output, borderwidth=NULL)) - 2)/
-            as.numeric(tkfont.measure(tkcget(.output, font=NULL), "0"))    
+            as.numeric(tkfont.measure(tkcget(.output, font=NULL), "0"))
         eval(parse(text=paste("options(width=", floor(width), ")", sep="")))
         }
     if (!getRcmdr("suppress.X11.warnings")){
@@ -521,30 +629,50 @@ doItAndPrint <- function(command, log=TRUE) {
         close(output.connection)
         }, add=TRUE)
     if (log) logger(command)
-    result <-  try(eval(parse(text=command), envir=.GlobalEnv), silent=TRUE)
-    if (class(result)[1] ==  "try-error"){
-        Message(message=paste(strsplit(result, ":")[[1]][2]), type="error")
-        if (.console.output) sink(type="output")
-        tkfocus(CommanderWindow())
-        return()
+    result <- try(parse(text=paste(command)), silent=TRUE)
+    if (class(result)[1] == "try-error"){
+    	Message(message=paste(strsplit(result, ":")[[1]][2]), type="error")
+    	if (.console.output) sink(type="output")
+    	tkfocus(CommanderWindow())
+    	return()
+    } else {
+        exprs <- result
+        result <- NULL
+    }
+    for (i in seq_along(exprs)) {
+        ei <- exprs[i]
+        result <-  try(withVisible(eval(ei, envir=.GlobalEnv)), silent=TRUE)
+	if (class(result)[1] ==  "try-error"){
+	    Message(message=paste(strsplit(result, ":")[[1]][2]), type="error")
+	    if (.console.output) sink(type="output")
+	    tkfocus(CommanderWindow())
+	    return()
+	    }
+	result <- if (result$visible == FALSE) NULL else result$value
+	if (!is.null(result)) pushOutput(result)
+	if (isS4object(result)) show(result) else print(result)
+	.Output <- readLines(output.connection)
+	if (length(.Output) > 0 && .Output[length(.Output)] == "NULL")
+	    .Output <- .Output[-length(.Output)] # suppress "NULL" line at end of output
+	if (length(.Output) != 0) {  # is there output to print?
+	    if (.console.output) {
+		out <- .Output
+		sink(type="output")
+		for (line in out) cat(paste(line, "\n", sep=""))
+		}
+	    else{
+		for (line in .Output) tkinsert(.output, "end", paste(line, "\n", sep=""))
+		tkyview.moveto(.output, 1)
+		}
+	    }
+	else if (.console.output) sink(type="output")
+	###### added by EN  ######################
+	    if (RExcelSupported())
+	    putRExcel(".rexcel.last.output",.Output)
+	###### end of change  #####################
+        # errors already intercepted, display any warnings
+        checkWarnings(readLines(messages.connection))
         }
-    if (isS4object(result)) show(result) else print(result)
-    .Output <- readLines(output.connection)
-    if (.Output[length(.Output)] == "NULL") .Output <- .Output[-length(.Output)] # suppress "NULL" line at end of output
-    if (length(.Output) != 0) {  # is there output to print?
-        if (.console.output) {
-            out <- .Output
-            sink(type="output")
-            for (line in out) cat(paste(line, "\n", sep=""))
-            }
-        else{
-            for (line in .Output) tkinsert(.output, "end", paste(line, "\n", sep=""))
-            tkyview.moveto(.output, 1)
-            }
-        }
-    else if (.console.output) sink(type="output")
-    # errors already intercepted, display any warnings
-    checkWarnings(readLines(messages.connection))
     result
     }
 
@@ -558,17 +686,17 @@ checkWarnings <- function(messages){
             if (length(messages) == 0) Message()
             else if (length(messages) > 10) {
                 messages <- c(paste(length(messages), "warnings."),
-                    gettextRcmdr("First and last 5 warnings:"), 
+                    gettextRcmdr("First and last 5 warnings:"),
                         head(messages,5), ". . .", tail(messages, 5))
                 Message(message=paste(messages, collapse="\n"), type="warning")
                 }
         else Message(message=paste(messages, collapse="\n"), type="warning")
-        }                        
+        }
     else{
         if (length(messages) == 0) Message()
-        else if (length(messages) > 10){ 
+        else if (length(messages) > 10){
             messages <- c(paste(length(messages), "warnings."),
-                gettextRcmdr("First and last 5 warnings:"), 
+                gettextRcmdr("First and last 5 warnings:"),
                     head(messages, 5), ". . .", tail(messages, 5))
             Message(message=paste(messages, collapse="\n"), type="warning")
             }
@@ -576,7 +704,7 @@ checkWarnings <- function(messages){
         }
     tkfocus(CommanderWindow())
     }
-    
+
 Message <- function(message, type=c("note", "error", "warning")){
     if (is.SciViews()) return(svMessage(message, type))    # +PhG
     .message <- MessagesWindow()
@@ -605,6 +733,10 @@ Message <- function(message, type=c("note", "error", "warning")){
         }
     putRcmdr("last.message", type)
     message <- paste(prefix, ": ", message, sep="")
+    ######### added by EN #####################
+   	if (RExcelSupported())
+    	putRExcel(".rexcel.last.message",message)
+    ######### end of change ###############
     lines <- strsplit(message, "\n")[[1]]
     for (line in lines){
         tagName <- messageTag()
@@ -624,3 +756,16 @@ messageTag <- function(reset=FALSE){
     putRcmdr("tagNumber", tagNumber)
     paste("message", tagNumber, sep="")
     }
+
+pushOutput <- function(element) {
+    stack <- getRcmdr("outputStack")
+    stack <- c(list(element), stack[-getRcmdr("length.output.stack")])
+    putRcmdr("outputStack", stack)
+    }
+
+popOutput <- function(){
+    stack <- getRcmdr("outputStack")
+    lastOutput <- stack[[1]]
+    putRcmdr("outputStack", c(stack[-1], NA))
+    lastOutput
+    }
